@@ -25,34 +25,35 @@ import (
 
 // Interpreter node structure for AST and CFG.
 type node struct {
-	debug  *nodeDebugData // debug info
-	child  []*node        // child subtrees (AST)
-	anc    *node          // ancestor (AST)
-	param  []*itype       // generic parameter nodes (AST)
-	start  *node          // entry point in subtree (CFG)
-	tnext  *node          // true branch successor (CFG)
-	fnext  *node          // false branch successor (CFG)
-	interp *Interpreter   // interpreter context
-	frame  *frame         // frame pointer used for closures only (TODO: suppress this)
-	index  int64          // node index (dot display)
-	findex int            // index of value in frame or frame size (func def, type def)
-	level  int            // number of frame indirections to access value
-	nleft  int            // number of children in left part (assign) or indicates preceding type (compositeLit)
-	nright int            // number of children in right part (assign)
-	kind   nkind          // kind of node
-	pos    token.Pos      // position in source code, relative to fset
-	sym    *symbol        // associated symbol
-	typ    *itype         // type of value in frame, or nil
-	recv   *receiver      // method receiver node for call, or nil
-	types  []reflect.Type // frame types, used by function literals only
-	scope  *scope         // frame scope
-	action action         // action
-	exec   bltn           // generated function to execute
-	gen    bltnGenerator  // generator function to produce above bltn
-	val    interface{}    // static generic value (CFG execution)
-	rval   reflect.Value  // reflection value to let runtime access interpreter (CFG)
-	ident  string         // set if node is a var or func
-	meta   interface{}    // meta stores meta information between gta runs, like errors
+	debug      *nodeDebugData // debug info
+	child      []*node        // child subtrees (AST)
+	anc        *node          // ancestor (AST)
+	param      []*itype       // generic parameter nodes (AST)
+	start      *node          // entry point in subtree (CFG)
+	tnext      *node          // true branch successor (CFG)
+	fnext      *node          // false branch successor (CFG)
+	interp     *Interpreter   // interpreter context
+	frame      *frame         // frame pointer used for closures only (TODO: suppress this)
+	index      int64          // node index (dot display)
+	findex     int            // index of value in frame or frame size (func def, type def)
+	level      int            // number of frame indirections to access value
+	nleft      int            // number of children in left part (assign) or indicates preceding type (compositeLit)
+	nright     int            // number of children in right part (assign)
+	kind       nkind          // kind of node
+	pos        token.Pos      // position in source code, relative to fset
+	sym        *symbol        // associated symbol
+	typ        *itype         // type of value in frame, or nil
+	recv       *receiver      // method receiver node for call, or nil
+	types      []reflect.Type // frame types, used by function literals only
+	scope      *scope         // frame scope
+	action     action         // action
+	exec       bltn           // generated function to execute
+	gen        bltnGenerator  // generator function to produce above bltn
+	val        interface{}    // static generic value (CFG execution)
+	rval       reflect.Value  // reflection value to let runtime access interpreter (CFG)
+	ident      string         // set if node is a var or func
+	redeclared bool           // set if node is a redeclared variable (CFG)
+	meta       interface{}    // meta stores meta information between gta runs, like errors
 }
 
 func (n *node) shouldBreak() bool {
@@ -105,7 +106,7 @@ type receiver struct {
 type frame struct {
 	// id is an atomic counter used for cancellation, only accessed
 	// via newFrame/runid/setrunid/clone.
-	// Located at start of struct to ensure proper aligment.
+	// Located at start of struct to ensure proper alignment.
 	id uint64
 
 	debug *frameDebugData
@@ -183,14 +184,14 @@ type opt struct {
 	noRun        bool              // compile, but do not run
 	fastChan     bool              // disable cancellable chan operations
 	specialStdio bool              // allows os.Stdin, os.Stdout, os.Stderr to not be file descriptors
-	unrestricted bool              // allow use of non sandboxed symbols
+	unrestricted bool              // allow use of non-sandboxed symbols
 }
 
 // Interpreter contains global resources and state.
 type Interpreter struct {
-	// id is an atomic counter counter used for run cancellation,
+	// id is an atomic counter used for run cancellation,
 	// only accessed via runid/stop
-	// Located at start of struct to ensure proper alignment on 32 bit
+	// Located at start of struct to ensure proper alignment on 32-bit
 	// architectures.
 	id uint64
 
@@ -404,21 +405,24 @@ func New(options Options) *Interpreter {
 }
 
 const (
-	bltnAppend  = "append"
-	bltnCap     = "cap"
-	bltnClose   = "close"
-	bltnComplex = "complex"
-	bltnImag    = "imag"
-	bltnCopy    = "copy"
-	bltnDelete  = "delete"
-	bltnLen     = "len"
-	bltnMake    = "make"
-	bltnNew     = "new"
-	bltnPanic   = "panic"
-	bltnPrint   = "print"
-	bltnPrintln = "println"
-	bltnReal    = "real"
-	bltnRecover = "recover"
+	bltnAlignof  = "unsafe.Alignof"
+	bltnAppend   = "append"
+	bltnCap      = "cap"
+	bltnClose    = "close"
+	bltnComplex  = "complex"
+	bltnImag     = "imag"
+	bltnCopy     = "copy"
+	bltnDelete   = "delete"
+	bltnLen      = "len"
+	bltnMake     = "make"
+	bltnNew      = "new"
+	bltnOffsetof = "unsafe.Offsetof"
+	bltnPanic    = "panic"
+	bltnPrint    = "print"
+	bltnPrintln  = "println"
+	bltnReal     = "real"
+	bltnRecover  = "recover"
+	bltnSizeof   = "unsafe.Sizeof"
 )
 
 func initUniverse() *scope {
