@@ -1,11 +1,13 @@
 package interp_test
 
 import (
+	"bytes"
 	"go/build"
 	"io"
 	"os"
 	"os/exec"
 	"path/filepath"
+	"runtime"
 	"strings"
 	"testing"
 
@@ -14,6 +16,13 @@ import (
 	"github.com/traefik/yaegi/stdlib/unsafe"
 )
 
+// The following tests depend on an incompatible language change in go1.22, where `for` variables are now
+// defined in body (thus reallocated at each loop). We skip them until both supported versions behave the same.
+// We will remove this in Go1.23.
+var testsToSkipGo122 = map[string]bool{"closure9.go": true, "closure10.go": true, "closure11.go": true, "closure12.go": true}
+
+var go122 = strings.HasPrefix(runtime.Version(), "go1.22")
+
 func TestInterpConsistencyBuild(t *testing.T) {
 	if testing.Short() {
 		t.Skip("short mode")
@@ -37,6 +46,8 @@ func TestInterpConsistencyBuild(t *testing.T) {
 			file.Name() == "assign12.go" || // expect error
 			file.Name() == "assign15.go" || // expect error
 			file.Name() == "bad0.go" || // expect error
+			file.Name() == "break0.go" || // expect error
+			file.Name() == "cont3.go" || // expect error
 			file.Name() == "const9.go" || // expect error
 			file.Name() == "export1.go" || // non-main package
 			file.Name() == "export0.go" || // non-main package
@@ -46,6 +57,9 @@ func TestInterpConsistencyBuild(t *testing.T) {
 			file.Name() == "fun23.go" || // expect error
 			file.Name() == "fun24.go" || // expect error
 			file.Name() == "fun25.go" || // expect error
+			file.Name() == "gen7.go" || // expect error
+			file.Name() == "gen8.go" || // expect error
+			file.Name() == "gen9.go" || // expect error
 			file.Name() == "if2.go" || // expect error
 			file.Name() == "import6.go" || // expect error
 			file.Name() == "init1.go" || // expect error
@@ -84,6 +98,7 @@ func TestInterpConsistencyBuild(t *testing.T) {
 			file.Name() == "redeclaration-global5.go" || // expect error
 			file.Name() == "redeclaration-global6.go" || // expect error
 			file.Name() == "redeclaration-global7.go" || // expect error
+			file.Name() == "panic0.go" || // expect error
 			file.Name() == "pkgname0.go" || // has deps
 			file.Name() == "pkgname1.go" || // expect error
 			file.Name() == "pkgname2.go" || // has deps
@@ -104,6 +119,7 @@ func TestInterpConsistencyBuild(t *testing.T) {
 			file.Name() == "range9.go" || // expect error
 			file.Name() == "unsafe6.go" || // needs go.mod to be 1.17
 			file.Name() == "unsafe7.go" || // needs go.mod to be 1.17
+			file.Name() == "type24.go" || // expect error
 			file.Name() == "type27.go" || // expect error
 			file.Name() == "type28.go" || // expect error
 			file.Name() == "type29.go" || // expect error
@@ -113,6 +129,13 @@ func TestInterpConsistencyBuild(t *testing.T) {
 			file.Name() == "type33.go" { // expect error
 			continue
 		}
+		// Skip some tests which are problematic in go1.21 only.
+		if go121 && testsToSkipGo121[file.Name()] {
+			continue
+		}
+		if go122 && testsToSkipGo122[file.Name()] {
+			continue
+		}
 
 		file := file
 		t.Run(file.Name(), func(t *testing.T) {
@@ -170,7 +193,7 @@ func TestInterpConsistencyBuild(t *testing.T) {
 				t.Fatal(err)
 			}
 
-			if string(outInterp) != string(outRun) {
+			if !bytes.Equal(outInterp, outRun) {
 				t.Errorf("\nGot: %q,\n want: %q", string(outInterp), string(outRun))
 			}
 		})
@@ -181,66 +204,78 @@ func TestInterpErrorConsistency(t *testing.T) {
 	testCases := []struct {
 		fileName       string
 		expectedInterp string
+		expectedStderr string
 		expectedExec   string
 	}{
 		{
 			fileName:       "assign11.go",
 			expectedInterp: "6:2: assignment mismatch: 3 variables but fmt.Println returns 2 values",
-			expectedExec:   "6:10: assignment mismatch: 3 variables but fmt.Println returns 2 values",
+			expectedExec:   "6:12: assignment mismatch: 3 variables but fmt.Println returns 2 values",
 		},
 		{
 			fileName:       "assign12.go",
 			expectedInterp: "6:2: assignment mismatch: 3 variables but fmt.Println returns 2 values",
-			expectedExec:   "6:10: assignment mismatch: 3 variables but fmt.Println returns 2 values",
+			expectedExec:   "6:13: assignment mismatch: 3 variables but fmt.Println returns 2 values",
 		},
 		{
 			fileName:       "bad0.go",
 			expectedInterp: "1:1: expected 'package', found println",
 			expectedExec:   "1:1: expected 'package', found println",
 		},
+		{
+			fileName:       "break0.go",
+			expectedInterp: "15:5: invalid break label OuterLoop",
+			expectedExec:   "15:11: invalid break label OuterLoop",
+		},
+		{
+			fileName:       "cont3.go",
+			expectedInterp: "15:5: invalid continue label OuterLoop",
+			expectedExec:   "15:14: invalid continue label OuterLoop",
+		},
 		{
 			fileName:       "const9.go",
 			expectedInterp: "5:2: constant definition loop",
-			expectedExec:   "5:2: constant definition loop",
+			expectedExec:   "5:2: initialization",
 		},
 		{
 			fileName:       "if2.go",
 			expectedInterp: "7:5: non-bool used as if condition",
-			expectedExec:   "7:2: non-bool i % 1000000 (type int) used as if condition",
+			expectedExec:   "7:5: non-boolean condition in if statement",
 		},
 		{
 			fileName:       "for7.go",
 			expectedInterp: "4:14: non-bool used as for condition",
-			expectedExec:   "4:2: non-bool i (type int) used as for condition",
+			expectedExec:   "4:14: non-boolean condition in for statement",
 		},
 		{
 			fileName:       "fun21.go",
 			expectedInterp: "4:2: not enough arguments to return",
-			expectedExec:   "4:2: not enough arguments to return",
+			expectedExec:   "4:2: not enough return values",
 		},
 		{
 			fileName:       "fun22.go",
 			expectedInterp: "6:2: not enough arguments in call to time.Date",
-			expectedExec:   "6:11: not enough arguments in call to time.Date",
+			expectedExec:   "6:2: not enough arguments in call to time.Date",
 		},
 		{
 			fileName:       "fun23.go",
 			expectedInterp: "3:17: too many arguments to return",
-			expectedExec:   "3:17: too many arguments to return",
+			expectedExec:   "3:24: too many return values",
 		},
 		{
 			fileName:       "issue-1093.go",
 			expectedInterp: "9:6: cannot use type untyped string as type int in assignment",
-			expectedExec:   `9:4: cannot use "a" + b() (type string) as type int in assignment`,
+			expectedExec:   `9:6: cannot use "a" + b() (value of type string)`,
 		},
 		{
 			fileName:       "op1.go",
 			expectedInterp: "5:2: invalid operation: mismatched types int and untyped float",
-			expectedExec:   "5:4: constant 1.3 truncated to integer",
+			expectedExec:   "5:7: 1.3 (untyped float constant) truncated to int",
 		},
 		{
 			fileName:       "bltn0.go",
 			expectedInterp: "4:7: use of builtin println not in function call",
+			expectedExec:   "4:7: println (built-in) must be called",
 		},
 		{
 			fileName:       "import6.go",
@@ -255,29 +290,40 @@ func TestInterpErrorConsistency(t *testing.T) {
 		{
 			fileName:       "switch9.go",
 			expectedInterp: "9:3: cannot fallthrough in type switch",
-			expectedExec:   "9:3: cannot fallthrough in type switch",
+			expectedExec:   "fallthrough",
 		},
 		{
 			fileName:       "switch13.go",
 			expectedInterp: "9:2: i is not a type",
-			expectedExec:   "9:2: i (type interface {}) is not a type",
+			expectedExec:   "9:7: i (variable of type interface{}) is not a type",
 		},
 		{
 			fileName:       "switch19.go",
 			expectedInterp: "37:2: duplicate case Bir in type switch",
-			expectedExec:   "37:2: duplicate case Bir in type switch",
+			expectedExec:   "37:7: duplicate case Bir in type switch",
+		},
+		{
+			fileName:       "panic0.go",
+			expectedInterp: "stop!",
+			expectedStderr: `
+../_test/panic0.go:16:2: panic: main.baz(...)
+../_test/panic0.go:12:2: panic: main.bar(...)
+../_test/panic0.go:8:2: panic: main.foo(...)
+../_test/panic0.go:4:2: panic: main.main(...)
+`,
 		},
 	}
 
 	for _, test := range testCases {
 		t.Run(test.fileName, func(t *testing.T) {
-			if len(test.expectedInterp) == 0 && len(test.expectedExec) == 0 {
-				t.Fatal("at least expectedInterp must be define")
+			if test.expectedInterp == "" && test.expectedExec == "" {
+				t.Fatal("at least expectedInterp must be defined")
 			}
 
 			filePath := filepath.Join("..", "_test", test.fileName)
 
-			i := interp.New(interp.Options{GoPath: build.Default.GOPATH})
+			var stderr bytes.Buffer
+			i := interp.New(interp.Options{GoPath: build.Default.GOPATH, Stderr: &stderr})
 			if err := i.Use(stdlib.Symbols); err != nil {
 				t.Fatal(err)
 			}
@@ -290,6 +336,12 @@ func TestInterpErrorConsistency(t *testing.T) {
 			if !strings.Contains(errEval.Error(), test.expectedInterp) {
 				t.Errorf("got %q, want: %q", errEval.Error(), test.expectedInterp)
 			}
+			if test.expectedStderr != "" {
+				exp, got := strings.TrimSpace(test.expectedStderr), strings.TrimSpace(stderr.String())
+				if exp != got {
+					t.Errorf("got %q, want: %q", got, exp)
+				}
+			}
 
 			cmd := exec.Command("go", "run", filePath)
 			outRun, errExec := cmd.CombinedOutput()
@@ -298,7 +350,7 @@ func TestInterpErrorConsistency(t *testing.T) {
 				t.Fatal("An error is expected but got none.")
 			}
 
-			if len(test.expectedExec) == 0 && !strings.Contains(string(outRun), test.expectedInterp) {
+			if test.expectedExec == "" && !strings.Contains(string(outRun), test.expectedInterp) {
 				t.Errorf("got %q, want: %q", string(outRun), test.expectedInterp)
 			} else if !strings.Contains(string(outRun), test.expectedExec) {
 				t.Errorf("got %q, want: %q", string(outRun), test.expectedExec)
