@@ -180,6 +180,27 @@ var errAbortHandler = errors.New("net/http: abort Handler")
 
 // Functions set to run during execution of CFG.
 
+func panicFunc(s *scope) string {
+	if s == nil {
+		return ""
+	}
+	def := s.def
+	if def == nil {
+		return s.pkgID
+	}
+	switch def.kind {
+	case funcDecl:
+		if c := def.child[1]; c.kind == identExpr {
+			return s.pkgID + "." + c.ident
+		}
+	case funcLit:
+		if def.anc != nil {
+			return panicFunc(def.anc.scope) + ".func"
+		}
+	}
+	return s.pkgID
+}
+
 // runCfg executes a node AST by walking its CFG and running node builtin at each step.
 func runCfg(n *node, f *frame, funcNode, callNode *node) {
 	var exec bltn
@@ -199,7 +220,7 @@ func runCfg(n *node, f *frame, funcNode, callNode *node) {
 			// suppress the logging here accordingly, to get a similar and consistent
 			// behavior.
 			if !ok || errorer.Error() != errAbortHandler.Error() {
-				fmt.Fprintln(n.interp.stderr, oNode.cfgErrorf("panic"))
+				fmt.Fprintln(n.interp.stderr, oNode.cfgErrorf("panic: %s(...)", panicFunc(oNode.scope)))
 			}
 			f.mutex.Unlock()
 			panic(f.recovered)
@@ -290,7 +311,7 @@ func typeAssert(n *node, withResult, withOk bool) {
 
 	typ := c1.typ // type to assert or convert to
 	typID := typ.id()
-	rtype := typ.rtype // type to assert
+	rtype := typ.refType(nil) // type to assert
 	next := getExec(n.tnext)
 
 	switch {
@@ -636,6 +657,15 @@ func assignFromCall(n *node) {
 				continue
 			}
 			s := f.data[ncall.findex+i]
+			c := n.child[i]
+			if n.kind == defineXStmt && !c.redeclared {
+				// Recreate destination value in case of define statement,
+				// to preserve previous value possibly in use by a closure.
+				data := getFrame(f, c.level).data
+				data[c.findex] = reflect.New(data[c.findex].Type()).Elem()
+				data[c.findex].Set(s)
+				continue
+			}
 			v(f).Set(s)
 		}
 		return next
@@ -1146,23 +1176,10 @@ func checkFieldIndex(typ reflect.Type, index []int) bool {
 
 func call(n *node) {
 	goroutine := n.anc.kind == goStmt
-	var method bool
 	c0 := n.child[0]
 	value := genValue(c0)
 	var values []func(*frame) reflect.Value
 
-	recvIndexLater := false
-	switch {
-	case c0.recv != nil:
-		// Compute method receiver value.
-		values = append(values, genValueRecv(c0))
-		method = true
-	case c0.action == aMethod:
-		// Add a place holder for interface method receiver.
-		values = append(values, nil)
-		method = true
-	}
-
 	numRet := len(c0.typ.ret)
 	variadic := variadicPos(n)
 	child := n.child[1:]
@@ -1262,10 +1279,6 @@ func call(n *node) {
 	if n.anc.kind == deferStmt {
 		// Store function call in frame for deferred execution.
 		value = genFunctionWrapper(c0)
-		if method {
-			// The receiver is already passed in the function wrapper, skip it.
-			values = values[1:]
-		}
 		n.exec = func(f *frame) bltn {
 			val := make([]reflect.Value, len(values)+1)
 			val[0] = value(f)
@@ -1301,11 +1314,6 @@ func call(n *node) {
 				callf = func(in []reflect.Value) []reflect.Value { return bf.Call(in) }
 			}
 
-			if method && len(values) > bf.Type().NumIn() {
-				// The receiver is already passed in the function wrapper, skip it.
-				values = values[1:]
-			}
-
 			if goroutine {
 				// Goroutine's arguments should be copied.
 				in := make([]reflect.Value, len(values))
@@ -1358,55 +1366,15 @@ func call(n *node) {
 		}
 
 		// Init variadic argument vector
-		varIndex := variadic
 		if variadic >= 0 {
-			if method {
-				vararg = nf.data[numRet+variadic+1]
-				varIndex++
-			} else {
-				vararg = nf.data[numRet+variadic]
-			}
+			vararg = nf.data[numRet+variadic]
 		}
 
 		// Copy input parameters from caller
 		if dest := nf.data[numRet:]; len(dest) > 0 {
 			for i, v := range values {
 				switch {
-				case method && i == 0:
-					// compute receiver
-					var src reflect.Value
-					if v == nil {
-						src = def.recv.val
-					} else {
-						src = v(f)
-						for src.IsValid() {
-							// traverse interface indirections to find out concrete type
-							vi, ok := src.Interface().(valueInterface)
-							if !ok {
-								break
-							}
-							src = vi.value
-						}
-					}
-					if recvIndexLater && def.recv != nil && len(def.recv.index) > 0 {
-						if src.Kind() == reflect.Ptr {
-							src = src.Elem().FieldByIndex(def.recv.index)
-						} else {
-							src = src.FieldByIndex(def.recv.index)
-						}
-					}
-					// Accommodate to receiver type
-					d := dest[0]
-					if ks, kd := src.Kind(), d.Kind(); ks != kd {
-						if kd == reflect.Ptr {
-							d.Set(src.Addr())
-						} else {
-							d.Set(src.Elem())
-						}
-					} else {
-						d.Set(src)
-					}
-				case variadic >= 0 && i >= varIndex:
+				case variadic >= 0 && i >= variadic:
 					if v(f).Type() == vararg.Type() {
 						vararg.Set(v(f))
 					} else {
@@ -1649,9 +1617,20 @@ func callBin(n *node) {
 				}
 				out := callFn(value(f), in)
 				for i, v := range rvalues {
-					if v != nil {
-						v(f).Set(out[i])
+					if v == nil {
+						continue // Skip assign "_".
 					}
+					c := n.anc.child[i]
+					if n.anc.kind == defineXStmt && !c.redeclared {
+						// In case of a define statement, the destination value in the frame
+						// must be recreated. This is necessary to preserve the previous value
+						// which may be still used in a separate closure.
+						data := getFrame(f, c.level).data
+						data[c.findex] = reflect.New(data[c.findex].Type()).Elem()
+						data[c.findex].Set(out[i])
+						continue
+					}
+					v(f).Set(out[i])
 				}
 				return tnext
 			}
@@ -1985,7 +1964,30 @@ func getMethodByName(n *node) {
 	l := n.level
 
 	n.exec = func(f *frame) bltn {
-		val := value0(f).Interface().(valueInterface)
+		// The interface object must be directly accessible, or embedded in a struct (exported anonymous field).
+		val0 := value0(f)
+		val, ok := value0(f).Interface().(valueInterface)
+		if !ok {
+			// Search the first embedded valueInterface.
+			for val0.Kind() == reflect.Ptr {
+				val0 = val0.Elem()
+			}
+			for i := 0; i < val0.NumField(); i++ {
+				fld := val0.Type().Field(i)
+				if !fld.Anonymous || !fld.IsExported() {
+					continue
+				}
+				if val, ok = val0.Field(i).Interface().(valueInterface); ok {
+					break
+					// TODO: should we keep track of all the vals that are indeed valueInterface,
+					// so that later on we can call MethodByName on all of them until one matches?
+				}
+			}
+			if !ok {
+				panic(n.cfgErrorf("invalid interface value %v", val0))
+			}
+		}
+		// Traverse nested interface values to get the concrete value.
 		for {
 			v, ok := val.value.Interface().(valueInterface)
 			if !ok {
@@ -2001,7 +2003,7 @@ func getMethodByName(n *node) {
 
 		typ := val.node.typ
 		if typ.node == nil && typ.cat == valueT {
-			// happens with a var of empty interface type, that has value of concrete type
+			// It happens with a var of empty interface type, that has value of concrete type
 			// from runtime, being asserted to "user-defined" interface.
 			if _, ok := typ.rtype.MethodByName(name); !ok {
 				panic(n.cfgErrorf("method not found: %s", name))
@@ -2009,27 +2011,12 @@ func getMethodByName(n *node) {
 			return next
 		}
 
-		m, li := typ.lookupMethod(name)
-
-		// Try harder to find a matching embedded valueInterface.
-		// TODO (marc): make sure it works for arbitrary depth and breadth.
-		if m == nil && isStruct(val.node.typ) {
-			v := val.value
-			for v.Type().Kind() == reflect.Ptr {
-				v = v.Elem()
-			}
-			nf := v.NumField()
-			for i := 0; i < nf; i++ {
-				var ok bool
-				if val, ok = v.Field(i).Interface().(valueInterface); !ok {
-					continue
-				}
-				if m, li = val.node.typ.lookupMethod(name); m != nil {
-					break
-				}
-			}
+		// Finally search method recursively in embedded valueInterfaces.
+		r, m, li := lookupMethodValue(val, name)
+		if r.IsValid() {
+			getFrame(f, l).data[i] = r
+			return next
 		}
-
 		if m == nil {
 			panic(n.cfgErrorf("method not found: %s", name))
 		}
@@ -2044,6 +2031,37 @@ func getMethodByName(n *node) {
 	}
 }
 
+// lookupMethodValue recursively looks within val for the method with the given
+// name. If a runtime value is found, it is returned in r, otherwise it is returned
+// in m, with li as the list of recursive field indexes.
+func lookupMethodValue(val valueInterface, name string) (r reflect.Value, m *node, li []int) {
+	if r = val.value.MethodByName(name); r.IsValid() {
+		return
+	}
+	if m, li = val.node.typ.lookupMethod(name); m != nil {
+		return
+	}
+	if !isStruct(val.node.typ) {
+		return
+	}
+	v := val.value
+	for v.Type().Kind() == reflect.Ptr {
+		v = v.Elem()
+	}
+	nf := v.NumField()
+	for i := 0; i < nf; i++ {
+		vi, ok := v.Field(i).Interface().(valueInterface)
+		if !ok {
+			continue
+		}
+		if r, m, li = lookupMethodValue(vi, name); m != nil {
+			li = append([]int{i}, li...)
+			return
+		}
+	}
+	return
+}
+
 func getIndexSeq(n *node) {
 	value := genValue(n.child[0])
 	index := n.val.([]int)
@@ -3886,31 +3904,44 @@ func slice0(n *node) {
 	}
 }
 
-func isNil(n *node) {
-	var value func(*frame) reflect.Value
-	c0 := n.child[0]
-	value = genValue(c0)
-	typ := n.typ.concrete().TypeOf()
-	isInterface := n.typ.TypeOf().Kind() == reflect.Interface
-	tnext := getExec(n.tnext)
-	dest := genValue(n)
-
-	if n.fnext == nil {
-		if !isInterfaceSrc(c0.typ) {
-			if isInterface {
+func isNilChild(child int) func(n *node) {
+	return func(n *node) {
+		var value func(*frame) reflect.Value
+		child := n.child[child]
+		value = genValue(child)
+		typ := n.typ.concrete().TypeOf()
+		isInterface := n.typ.TypeOf().Kind() == reflect.Interface
+		tnext := getExec(n.tnext)
+		dest := genValue(n)
+		if n.fnext == nil {
+			if !isInterfaceSrc(child.typ) {
+				if isInterface {
+					n.exec = func(f *frame) bltn {
+						dest(f).Set(reflect.ValueOf(value(f).IsNil()).Convert(typ))
+						return tnext
+					}
+					return
+				}
 				n.exec = func(f *frame) bltn {
-					dest(f).Set(reflect.ValueOf(value(f).IsNil()).Convert(typ))
+					dest(f).SetBool(value(f).IsNil())
 					return tnext
 				}
 				return
 			}
-			n.exec = func(f *frame) bltn {
-				dest(f).SetBool(value(f).IsNil())
-				return tnext
+			if isInterface {
+				n.exec = func(f *frame) bltn {
+					v := value(f)
+					var r bool
+					if vi, ok := v.Interface().(valueInterface); ok {
+						r = (vi == valueInterface{} || vi.node.kind == basicLit && vi.node.typ.cat == nilT)
+					} else {
+						r = v.IsNil()
+					}
+					dest(f).Set(reflect.ValueOf(r).Convert(typ))
+					return tnext
+				}
+				return
 			}
-			return
-		}
-		if isInterface {
 			n.exec = func(f *frame) bltn {
 				v := value(f)
 				var r bool
@@ -3919,55 +3950,43 @@ func isNil(n *node) {
 				} else {
 					r = v.IsNil()
 				}
-				dest(f).Set(reflect.ValueOf(r).Convert(typ))
+				dest(f).SetBool(r)
 				return tnext
 			}
 			return
 		}
+
+		fnext := getExec(n.fnext)
+
+		if !isInterfaceSrc(child.typ) {
+			n.exec = func(f *frame) bltn {
+				if value(f).IsNil() {
+					dest(f).SetBool(true)
+					return tnext
+				}
+				dest(f).SetBool(false)
+				return fnext
+			}
+			return
+		}
+
 		n.exec = func(f *frame) bltn {
 			v := value(f)
-			var r bool
 			if vi, ok := v.Interface().(valueInterface); ok {
-				r = (vi == valueInterface{} || vi.node.kind == basicLit && vi.node.typ.cat == nilT)
-			} else {
-				r = v.IsNil()
+				if (vi == valueInterface{} || vi.node.kind == basicLit && vi.node.typ.cat == nilT) {
+					dest(f).SetBool(true)
+					return tnext
+				}
+				dest(f).SetBool(false)
+				return fnext
 			}
-			dest(f).SetBool(r)
-			return tnext
-		}
-		return
-	}
-
-	fnext := getExec(n.fnext)
-
-	if !isInterfaceSrc(c0.typ) {
-		n.exec = func(f *frame) bltn {
-			if value(f).IsNil() {
+			if v.IsNil() {
 				dest(f).SetBool(true)
 				return tnext
 			}
 			dest(f).SetBool(false)
 			return fnext
 		}
-		return
-	}
-
-	n.exec = func(f *frame) bltn {
-		v := value(f)
-		if vi, ok := v.Interface().(valueInterface); ok {
-			if (vi == valueInterface{} || vi.node.kind == basicLit && vi.node.typ.cat == nilT) {
-				dest(f).SetBool(true)
-				return tnext
-			}
-			dest(f).SetBool(false)
-			return fnext
-		}
-		if v.IsNil() {
-			dest(f).SetBool(true)
-			return tnext
-		}
-		dest(f).SetBool(false)
-		return fnext
 	}
 }
 
