@@ -484,6 +484,7 @@ func nodeType2(interp *Interpreter, sc *scope, n *node, seen []*node) (t *itype,
 			length = int(vInt(sym.rval))
 		default:
 			// Size is defined by a numeric constant expression.
+			var ok bool
 			if _, err := interp.cfg(c0, sc, sc.pkgID, sc.pkgName); err != nil {
 				if strings.Contains(err.Error(), " undefined: ") {
 					incomplete = true
@@ -491,12 +492,17 @@ func nodeType2(interp *Interpreter, sc *scope, n *node, seen []*node) (t *itype,
 				}
 				return nil, err
 			}
-			v, ok := c0.rval.Interface().(constant.Value)
-			if !ok {
-				incomplete = true
-				break
+			if !c0.rval.IsValid() {
+				return nil, c0.cfgErrorf("undefined array size")
+			}
+			if length, ok = c0.rval.Interface().(int); !ok {
+				v, ok := c0.rval.Interface().(constant.Value)
+				if !ok {
+					incomplete = true
+					break
+				}
+				length = constToInt(v)
 			}
-			length = constToInt(v)
 		}
 		val, err := nodeType2(interp, sc, n.child[1], seen)
 		if err != nil {
@@ -983,16 +989,18 @@ func nodeType2(interp *Interpreter, sc *scope, n *node, seen []*node) (t *itype,
 					rtype = rtype.Elem()
 				}
 				t = valueTOf(rtype, withNode(n), withScope(sc))
-			} else {
-				err = n.cfgErrorf("undefined selector %s.%s", lt.path, name)
+				break
 			}
+			// Continue search in source package, as it may exist if package contains generics.
+			fallthrough
 		case srcPkgT:
-			pkg := interp.srcPkg[lt.path]
-			if s, ok := pkg[name]; ok {
-				t = s.typ
-			} else {
-				err = n.cfgErrorf("undefined selector %s.%s", lt.path, name)
+			if pkg, ok := interp.srcPkg[lt.path]; ok {
+				if s, ok := pkg[name]; ok {
+					t = s.typ
+					break
+				}
 			}
+			err = n.cfgErrorf("undefined selector %s.%s", lt.path, name)
 		default:
 			if m, _ := lt.lookupMethod(name); m != nil {
 				t, err = nodeType2(interp, sc, m.child[2], seen)
@@ -1087,6 +1095,9 @@ func nodeType2(interp *Interpreter, sc *scope, n *node, seen []*node) (t *itype,
 			sc.sym[sname].typ = t
 		}
 
+	case typeAssertExpr:
+		t, err = nodeType2(interp, sc, n.child[1], seen)
+
 	default:
 		err = n.cfgErrorf("type definition not implemented: %s", n.kind)
 	}
@@ -1221,6 +1232,8 @@ func fieldName(n *node) string {
 		return fieldName(n.child[1])
 	case starExpr:
 		return fieldName(n.child[0])
+	case indexExpr:
+		return fieldName(n.child[0])
 	case identExpr:
 		return n.ident
 	default:
@@ -1336,7 +1349,7 @@ func (t *itype) numOut() int {
 		}
 	case builtinT:
 		switch t.name {
-		case "append", "cap", "complex", "copy", "imag", "len", "make", "new", "real", "recover":
+		case "append", "cap", "complex", "copy", "imag", "len", "make", "new", "real", "recover", "unsafe.Alignof", "unsafe.Offsetof", "unsafe.Sizeof":
 			return 1
 		}
 	}
@@ -1522,7 +1535,7 @@ func (t *itype) ordered() bool {
 	return isInt(typ) || isFloat(typ) || isString(typ)
 }
 
-// Equals returns true if the given type is identical to the receiver one.
+// equals returns true if the given type is identical to the receiver one.
 func (t *itype) equals(o *itype) bool {
 	switch ti, oi := isInterface(t), isInterface(o); {
 	case ti && oi:
@@ -1536,13 +1549,21 @@ func (t *itype) equals(o *itype) bool {
 	}
 }
 
+// matchDefault returns true if the receiver default type is the same as the given one.
+func (t *itype) matchDefault(o *itype) bool {
+	return t.untyped && t.id() == "untyped "+o.id()
+}
+
 // MethodSet defines the set of methods signatures as strings, indexed per method name.
 type methodSet map[string]string
 
 // Contains returns true if the method set m contains the method set n.
 func (m methodSet) contains(n methodSet) bool {
-	for k, v := range n {
-		if m[k] != v {
+	for k := range n {
+		// Only check the presence of method, not its complete signature,
+		// as the receiver may be part of the arguments, which makes a
+		// robust check complex.
+		if _, ok := m[k]; !ok {
 			return false
 		}
 	}
@@ -1705,6 +1726,7 @@ func (t *itype) fieldSeq(seq []int) *itype {
 func (t *itype) lookupField(name string) []int {
 	seen := map[*itype]bool{}
 	var lookup func(*itype) []int
+	tias := isStruct(t)
 
 	lookup = func(typ *itype) []int {
 		if seen[typ] {
@@ -1723,6 +1745,11 @@ func (t *itype) lookupField(name string) []int {
 		for i, f := range typ.field {
 			switch f.typ.cat {
 			case ptrT, structT, interfaceT, linkedT:
+				if tias != isStruct(f.typ) {
+					// Interface fields are not valid embedded struct fields.
+					// Struct fields are not valid interface fields.
+					break
+				}
 				if index2 := lookup(f.typ); len(index2) > 0 {
 					return append([]int{i}, index2...)
 				}
@@ -1832,6 +1859,39 @@ func (t *itype) lookupMethod2(name string, seen map[*itype]bool) (*node, []int)
 	return m, index
 }
 
+// interfaceMethod returns type of method matching an interface method name (not as a concrete method).
+func (t *itype) interfaceMethod(name string) *itype {
+	return t.interfaceMethod2(name, nil)
+}
+
+func (t *itype) interfaceMethod2(name string, seen map[*itype]bool) *itype {
+	if seen == nil {
+		seen = map[*itype]bool{}
+	}
+	if seen[t] {
+		return nil
+	}
+	seen[t] = true
+	if t.cat == ptrT {
+		return t.val.interfaceMethod2(name, seen)
+	}
+	for _, f := range t.field {
+		if f.name == name && isInterface(t) {
+			return f.typ
+		}
+		if !f.embed {
+			continue
+		}
+		if typ := f.typ.interfaceMethod2(name, seen); typ != nil {
+			return typ
+		}
+	}
+	if t.cat == linkedT || isInterfaceSrc(t) && t.val != nil {
+		return t.val.interfaceMethod2(name, seen)
+	}
+	return nil
+}
+
 // methodDepth returns a depth greater or equal to 0, or -1 if no match.
 func (t *itype) methodDepth(name string) int {
 	if m, lint := t.lookupMethod(name); m != nil {
@@ -1919,11 +1979,6 @@ var (
 	constVal           = reflect.TypeOf((*constant.Value)(nil)).Elem()
 )
 
-type fieldRebuild struct {
-	typ *itype
-	idx int
-}
-
 type refTypeContext struct {
 	defined map[string]*itype
 
@@ -1931,7 +1986,7 @@ type refTypeContext struct {
 	// type name (as key) is used as a field of another (or possibly the same) struct
 	// type. Each of these fields will then live as an unsafe2.dummy type until the
 	// whole recursion is fully resolved, and the type is fixed.
-	refs map[string][]fieldRebuild
+	refs map[string][]*itype
 
 	// When we detect for the first time that we are in a recursive type (thanks to
 	// defined), we keep track of the first occurrence of the type where the recursion
@@ -2004,7 +2059,7 @@ func (t *itype) refType(ctx *refTypeContext) reflect.Type {
 	if ctx == nil {
 		ctx = &refTypeContext{
 			defined: map[string]*itype{},
-			refs:    map[string][]fieldRebuild{},
+			refs:    map[string][]*itype{},
 		}
 	}
 	if t.incomplete || t.cat == nilT {
@@ -2029,14 +2084,14 @@ func (t *itype) refType(ctx *refTypeContext) reflect.Type {
 
 		// The recursion has not been fully resolved yet.
 		// To indicate that a rebuild is needed on the englobing struct,
-		// return a dummy field type and create an entry with an empty fieldRebuild.
+		// return a dummy field type and create an empty entry.
 		flds := ctx.refs[name]
 		ctx.rect = dt
 
 		// We know we are used as a field by someone, but we don't know by who
-		// at this point in the code, so we just mark it as an empty fieldRebuild for now.
-		// We'll complete the fieldRebuild in the caller.
-		ctx.refs[name] = append(flds, fieldRebuild{})
+		// at this point in the code, so we just mark it as an empty *itype for now.
+		// We'll complete the *itype in the caller.
+		ctx.refs[name] = append(flds, (*itype)(nil))
 		return unsafe2.DummyType
 	}
 	if isGeneric(t) {
@@ -2093,23 +2148,33 @@ func (t *itype) refType(ctx *refTypeContext) reflect.Type {
 		}
 		ctx.slevel++
 		var fields []reflect.StructField
-		for i, f := range t.field {
+		for _, f := range t.field {
 			field := reflect.StructField{
-				Name: exportName(f.name), Type: f.typ.refType(ctx),
-				Tag: reflect.StructTag(f.tag), Anonymous: f.embed,
+				Name: exportName(f.name),
+				Type: f.typ.refType(ctx),
+				Tag:  reflect.StructTag(f.tag),
+			}
+			if len(t.field) == 1 && f.embed {
+				// Mark the field as embedded (anonymous) only if it is the
+				// only one, to avoid a panic due to golang/go#15924 issue.
+				field.Anonymous = true
 			}
 			fields = append(fields, field)
 			// Find any nil type refs that indicates a rebuild is needed on this field.
 			for _, flds := range ctx.refs {
 				for j, fld := range flds {
-					if fld.typ == nil {
-						flds[j] = fieldRebuild{typ: t, idx: i}
+					if fld == nil {
+						flds[j] = t
 					}
 				}
 			}
 		}
 		ctx.slevel--
-		fieldFix := []int{} // Slice of field indices to fix for recursivity.
+		type fixStructField struct {
+			name  string
+			index int
+		}
+		fieldFix := []fixStructField{} // Slice of field indices to fix for recursivity.
 		t.rtype = reflect.StructOf(fields)
 		if ctx.isComplete() {
 			for _, s := range ctx.defined {
@@ -2118,8 +2183,12 @@ func (t *itype) refType(ctx *refTypeContext) reflect.Type {
 					if strings.HasSuffix(f.Type.String(), "unsafe2.dummy") {
 						unsafe2.SetFieldType(s.rtype, i, ctx.rect.fixDummy(s.rtype.Field(i).Type))
 						if name == s.path+"/"+s.name {
-							fieldFix = append(fieldFix, i)
+							fieldFix = append(fieldFix, fixStructField{s.name, i})
 						}
+						continue
+					}
+					if f.Type.Kind() == reflect.Func && strings.Contains(f.Type.String(), "unsafe2.dummy") {
+						fieldFix = append(fieldFix, fixStructField{s.name, i})
 					}
 				}
 			}
@@ -2134,9 +2203,11 @@ func (t *itype) refType(ctx *refTypeContext) reflect.Type {
 		// and we need both the loop above, around all the struct fields, and the loop
 		// below, around the ctx.refs.
 		for _, f := range ctx.refs[name] {
-			for _, index := range fieldFix {
-				ftyp := f.typ.field[index].typ.refType(&refTypeContext{defined: ctx.defined, rebuilding: true})
-				unsafe2.SetFieldType(f.typ.rtype, index, ftyp)
+			for _, ff := range fieldFix {
+				if ff.name == f.name {
+					ftyp := f.field[ff.index].typ.refType(&refTypeContext{defined: ctx.defined, rebuilding: true})
+					unsafe2.SetFieldType(f.rtype, ff.index, ftyp)
+				}
 			}
 		}
 	default:
